import datetime
import json
import warnings
from typing import TypeVar

import requests
import urllib3.util

from common.ClientProfileMain import ClientProfile
from common.HttpProfileMain import HttpProfile
from common.constants import AICloudConstants
from common.exception.AICloudExceptionMain import InvalidArgumentException, AICloudException
from common.signer.AICloudClientSignerMain import AICloudClientSigner

T = TypeVar('T')

class AICloudRestTemplate(object):
    httpProfile: HttpProfile
    clientProfile: ClientProfile

    def __init__(self, httpProfile: HttpProfile, clientProfile: ClientProfile):
        self.httpProfile = httpProfile
        self.clientProfile = clientProfile

    def getForResponse(self, urlStr: str, token: str, params: dict, body: dict):

        if self.httpProfile is None:
            raise InvalidArgumentException
        if self.clientProfile is None:
            raise InvalidArgumentException

        headers = {
            "Content-Type": "application/json"
        }
        if token is not None:
            headers["token"] = token

        url = self.httpProfile.protocol + "://" + self.httpProfile.host + ":" + str(self.httpProfile.port)

        finalParams: str = ""
        if params is not None:
            paramsKeys = list(params.keys())
            if len(paramsKeys) != 0:
                finalParams = "?"
                for pos in range(len(paramsKeys)):
                    if pos != len(paramsKeys):
                        finalParams = finalParams + "&" + paramsKeys[pos] + "=" + params.get(paramsKeys[pos])
                    else:
                        finalParams = finalParams + paramsKeys[pos] + "=" + params.get(paramsKeys[pos])
        path = "/" + self.httpProfile.contextPath + "/" + urlStr + finalParams

        return requests.get(url + path, data=body, headers=headers)

    def getForEntity(self, urlStr: str, token: str, params: dict, body: dict, clazz: T) -> T:
        response = self.getForResponse(urlStr=urlStr, token=token, params=params, body=body)
        return clazz(response.json())

    def postForResponse(self, url: str, headers: dict[str, str], body: dict):

        if not headers.keys().__contains__(AICloudConstants.X_AIC_VERSION):
            headers[AICloudConstants.X_AIC_VERSION] = AICloudConstants.VERSION

        if not headers.keys().__contains__(AICloudConstants.X_AIC_TIMESTAMP):
            headers[AICloudConstants.X_AIC_TIMESTAMP] = str(int(datetime.datetime.now().timestamp()))

        if not headers.keys().__contains__(AICloudConstants.X_AIC_SIGNED_HEADERS):
            headers[AICloudConstants.X_AIC_SIGNED_HEADERS] = AICloudConstants.MIN_HEADER_LIST

        if not headers.keys().__contains__("Content-Type"):
            headers["Content-Type"] = "application/json; charset=utf-8"

        parsedUrl = urllib3.util.parse_url(url)
        clientSigner = AICloudClientSigner(
            self.clientProfile.accessKey,
            self.clientProfile.secretAccessKey,
            "POST",
            parsedUrl.path,
            headers,
            None,
            AICloudConstants.MIN_HEADER_LIST,
            headers.get(AICloudConstants.X_AIC_SERVICE),
            json.dumps(body)
        )
        clientSignature = clientSigner.doSignature()
        headers[AICloudConstants.X_AIC_SIGNATURE] = clientSignature

        timeout = self.httpProfile.connectTimeout if self.httpProfile.readTimeout is None else (
        self.httpProfile.connectTimeout, self.httpProfile.readTimeout)
        try:
            return requests.post(url, json=body, headers=headers, verify=self.httpProfile.verifyCert, timeout=timeout)
        except BaseException as e:
            raise AICloudException(message=e.__str__())

    def postForEntity(self, urlStr: str, token: str, params: dict, body: dict, clazz: T) -> T:
        response = self.postForResponse(urlStr, token, params, body)
        ret = clazz(response.json())
        return ret
