import json
from typing import TypeVar

from common.ClientProfileMain import ClientProfile
from common.HttpProfileMain import HttpProfile
from client.AICloudRestTemplateMain import AICloudRestTemplate
from common.InquiryPriceResponseMain import InquiryPriceResponse
from common.constants import AICloudConstants
from common.exception.AICloudExceptionMain import InvalidPasswordException, InvalidArgumentException
from common.utils import PasswordUtils
from ecs.DeleteInstancesRequestMain import DeleteInstancesRequest
from ecs.DeleteInstancesResponseMain import DeleteInstancesResponse
from ecs.DescribeInstancesRequestMain import DescribeInstancesRequest
from ecs.DescribeInstancesResponseMain import DescribeInstancesResponse
from ecs.InquiryPriceRunInstancesResponseMain import InquiryPriceRunInstancesResponse
from ecs.RebootInstancesRequestMain import RebootInstancesRequest
from ecs.RebootInstancesResponseMain import RebootInstancesResponse
from ecs.RenewInstancesRequestMain import RenewInstancesRequest
from ecs.RenewInstancesResponseMain import RenewInstancesResponse
from ecs.RunInstancesRequestMain import RunInstancesRequest
from ecs.RunInstancesResponseMain import RunInstancesResponse
from ecs.StartInstancesRequestMain import StartInstancesRequest
from ecs.StartInstancesResponseMain import StartInstancesResponse
from ecs.StopInstancesRequestMain import StopInstancesRequest
from ecs.StopInstancesResponseMain import StopInstancesResponse

T = TypeVar('T')


class ECSClient(object):
    httpProfile: HttpProfile
    clientProfile: ClientProfile

    client: AICloudRestTemplate

    def __init__(self, httpProfile: HttpProfile = None, clientProfile: ClientProfile = None):
        self.clientProfile = clientProfile
        if httpProfile is None:
            self.httpProfile = HttpProfile()
        else:
            self.httpProfile = httpProfile
        self.client = AICloudRestTemplate(self.httpProfile, self.clientProfile)

    def GenericRequest(self, request: dict, action: str, clazz: T) -> T:
        if self.httpProfile is None or self.clientProfile is None:
            raise InvalidArgumentException

        url: str = self.httpProfile.protocol + \
                   "://" + \
                   self.httpProfile.host + \
                   ":" + \
                   str(self.httpProfile.port) + \
                   "/" + \
                   self.httpProfile.contextPath + \
                   "/" + \
                   AICloudConstants.VERSION.lower() + \
                   "/" + \
                   AICloudConstants.ECS_SERVICE.lower() + \
                   "/" + \
                   action
        headers = dict()
        headers[AICloudConstants.X_AIC_ACTION] = action
        headers[AICloudConstants.X_AIC_ACCESSKEY] = self.clientProfile.accessKey
        headers["Host"] = self.httpProfile.host + ":" + str(self.httpProfile.port)
        # headers["Host"] = self.httpProfile.host + ":" + self.httpProfile.port
        headers[AICloudConstants.X_AIC_SERVICE] = AICloudConstants.ECS_SERVICE

        response = self.client.postForResponse(url, headers, body=request)
        jsonObj = response.json()
        return clazz(jsonObj)

    def DescribeInstances(self, request: DescribeInstancesRequest) -> DescribeInstancesResponse:
        if request.pageNum is not None:
            if request.pageNum <= 0:
                raise InvalidArgumentException
        return self.GenericRequest(request.__dict__, "DescribeInstances", DescribeInstancesResponse)

    def InquiryPriceRunInstances(self, request: RunInstancesRequest) -> InquiryPriceRunInstancesResponse:
        requestDict = request.__dict__
        eipObj = request.eip
        ebsArray = request.ebs
        requestDict["eip"] = eipObj.__dict__
        ebsList: list[dict] = []
        for ebs in ebsArray:
            ebsList.append(ebs.__dict__)
        requestDict["ebs"] = ebsList
        return self.GenericRequest(requestDict, "InquiryPriceRunInstances", InquiryPriceRunInstancesResponse)

    def RunInstances(self, request: RunInstancesRequest) -> RunInstancesResponse:
        if request.password is not None:
            rawPassword = PasswordUtils.fromHex(request.password)
            if not PasswordUtils.validPassword(rawPassword):
                raise InvalidPasswordException
            request.password = rawPassword

        requestDict = request.__dict__
        eipObj = request.eip
        ebsArray = request.ebs
        if eipObj is not None:
            requestDict["eip"] = eipObj.__dict__
        if ebsArray is not None:
            ebsList: list[dict] = []
            for ebs in ebsArray:
                ebsList.append(ebs.__dict__)
            requestDict["ebs"] = ebsList
        return self.GenericRequest(requestDict, "RunInstances",
                                   RunInstancesResponse)

    def StartInstances(self, request: StartInstancesRequest) -> StartInstancesResponse:
        return self.GenericRequest(request.__dict__, "StartInstances", StartInstancesResponse)

    def StopInstances(self, request: StopInstancesRequest) -> StopInstancesResponse:
        return self.GenericRequest(request.__dict__, "StopInstances", StopInstancesResponse)

    def RebootInstances(self, request: RebootInstancesRequest) -> RebootInstancesResponse:
        return self.GenericRequest(request.__dict__, "RebootInstances", RebootInstancesResponse)

    def DeleteInstance(self, request: DeleteInstancesRequest) -> DeleteInstancesResponse:
        return self.GenericRequest(request.__dict__, "DeleteInstances", DeleteInstancesResponse)

    def RenewInstances(self, request: RenewInstancesRequest) -> RenewInstancesResponse:
        return self.GenericRequest(request.__dict__, "RenewInstances", RenewInstancesResponse)
